# Framework constants
FRAMEWORK_NAME = 'FRAMEWORK_NAME'

# Database Constants
DATABASE_CONFIG = 'DATABASE_CONFIG'
DATABASE_NAME = 'DATABASE_NAME'
DATABASE_TYPE = 'DATABASE_TYPE'
DATABASE_DIRECTORY_PATH = 'DATABASE_DIRECTORY_PATH'
DATABASE_URL = 'DATABASE_URL'

RESOURCES_DIRECTORY = "RESOURCES_DIRECTORY"
LOG_LEVEL = 'LOG_LEVEL'
BASE_DIR = 'BASE_DIR'
SQLALCHEMY_DATABASE_URI = 'SQLALCHEMY_DATABASE_URI'


BINANCE = "BINANCE"

IDENTIFIER_QUERY_PARAM = "identifier"
TARGET_SYMBOL_QUERY_PARAM = "target_symbol"
TRADING_SYMBOL_QUERY_PARAM = "trading_symbol"
ORDER_SIDE_QUERY_PARAM = "side"
STATUS_QUERY_PARAM = "status"
POSITION_SYMBOL_QUERY_PARAM = "position"
SYMBOL_QUERY_PARAM = "symbol"
TIME_FRAME_QUERY_PARAM = "time_frame"

RESERVED_IDENTIFIERS = [
    BINANCE
]

BINANCE_API_KEY = "binance_api_key"
BINANCE_SECRET_KEY = "binance_secret_key"

TRADING_SYMBOL = "TRADING_SYMBOL"

CHECK_PENDING_ORDERS = "CHECK_PENDING_ORDERS"
